@@ -67,10 +67,10 @@ TODO:
  */
 
 #include "vim.h"
-#undef EXTERN			/* tcl.h defines it too */
+#undef EXTERN			// tcl.h defines it too
 
 #ifdef DYNAMIC_TCL
-# define USE_TCL_STUBS /* use tcl's stubs mechanism */
+# define USE_TCL_STUBS // use tcl's stubs mechanism
 #endif
 
 #include <tcl.h>
@@ -106,7 +106,7 @@ static tcl_info tclinfo = { NULL, 0, 0, 0, 0, NULL, NULL };
 #define VIMOUT	((ClientData)1)
 #define VIMERR	((ClientData)2)
 
-/* This appears to be new in Tcl 8.4. */
+// This appears to be new in Tcl 8.4.
 #ifndef CONST84
 # define CONST84
 #endif
@@ -123,9 +123,9 @@ struct ref
     struct ref	*next;
 
     Tcl_Interp	*interp;
-    Tcl_Command cmd;	    /* Tcl command that represents this object */
-    Tcl_Obj	*delcmd;    /* Tcl command to call when object is being del. */
-    void	*vimobj;    /* Vim window or buffer (win_T* or buf_T*) */
+    Tcl_Command cmd;	    // Tcl command that represents this object
+    Tcl_Obj	*delcmd;    // Tcl command to call when object is being del.
+    void	*vimobj;    // Vim window or buffer (win_T* or buf_T*)
 };
 static char * tclgetbuffer _ANSI_ARGS_((Tcl_Interp *interp, buf_T *buf));
 static char * tclgetwindow _ANSI_ARGS_((Tcl_Interp *interp, win_T *win));
@@ -142,11 +142,11 @@ static void tclmsg _ANSI_ARGS_((char *text));
 static void tclerrmsg _ANSI_ARGS_((char *text));
 static void tclupdatevars _ANSI_ARGS_((void));
 
-static struct ref refsdeleted;	/* dummy object for deleted ref list */
+static struct ref refsdeleted;	// dummy object for deleted ref list
 
-/*****************************************************************************
- * TCL interface manager
- ****************************************************************************/
+//////////////////////////////////////////////////////////////////////////////
+// TCL interface manager
+////////////////////////////////////////////////////////////////////////////
 
 #if defined(DYNAMIC_TCL) || defined(PROTO)
 # ifndef DYNAMIC_TCL_DLL
@@ -156,22 +156,24 @@ static struct ref refsdeleted;	/* dummy object for deleted ref list */
 #  define DYNAMIC_TCL_VER "8.3"
 # endif
 
-# ifndef  DYNAMIC_TCL /* Just generating prototypes */
+# ifndef  DYNAMIC_TCL // Just generating prototypes
 typedef int HANDLE;
 # endif
 
-# ifndef WIN3264
+# ifndef MSWIN
 #  include <dlfcn.h>
 #  define HANDLE void*
 #  define TCL_PROC void*
 #  define load_dll(n) dlopen((n), RTLD_LAZY|RTLD_GLOBAL)
 #  define symbol_from_dll dlsym
 #  define close_dll dlclose
+#  define load_dll_error dlerror
 # else
 #  define TCL_PROC FARPROC
 #  define load_dll vimLoadLib
 #  define symbol_from_dll GetProcAddress
 #  define close_dll FreeLibrary
+#  define load_dll_error GetWin32Error
 # endif
 
 /*
@@ -213,7 +215,7 @@ tcl_runtime_link_init(char *libname, int verbose)
     if (!(hTclLib = load_dll(libname)))
     {
 	if (verbose)
-	    EMSG2(_(e_loadlib), libname);
+	    semsg(_(e_could_not_load_library_str_str), libname, load_dll_error());
 	return FAIL;
     }
     for (i = 0; tcl_funcname_table[i].ptr; ++i)
@@ -224,13 +226,13 @@ tcl_runtime_link_init(char *libname, int verbose)
 	    close_dll(hTclLib);
 	    hTclLib = NULL;
 	    if (verbose)
-		EMSG2(_(e_loadfunc), tcl_funcname_table[i].name);
+		semsg(_(e_could_not_load_library_function_str), tcl_funcname_table[i].name);
 	    return FAIL;
 	}
     }
     return OK;
 }
-#endif /* defined(DYNAMIC_TCL) || defined(PROTO) */
+#endif // defined(DYNAMIC_TCL) || defined(PROTO)
 
 #ifdef DYNAMIC_TCL
 static char *find_executable_arg = NULL;
@@ -270,28 +272,32 @@ tcl_enabled(int verbose)
 		Tcl_DeleteInterp(interp);
 		stubs_initialized = TRUE;
 	    }
-	    /* FIXME: When Tcl_InitStubs() was failed, how delete interp? */
+	    // FIXME: When Tcl_InitStubs() was failed, how delete interp?
 	}
     }
     return stubs_initialized;
 }
+#endif
+
+#if defined(EXITFREE) || defined(PROTO)
+    void
+vim_tcl_finalize(void)
+{
+# ifdef DYNAMIC_TCL
+    if (stubs_initialized)
+# endif
+	Tcl_Finalize();
+}
 #endif
 
     void
 tcl_end(void)
 {
-#ifdef DYNAMIC_TCL
-    if (hTclLib)
-    {
-	close_dll(hTclLib);
-	hTclLib = NULL;
-    }
-#endif
 }
 
-/****************************************************************************
-  Tcl commands
- ****************************************************************************/
+/////////////////////////////////////////////////////////////////////////////
+// Tcl commands
+////////////////////////////////////////////////////////////////////////////
 
 /*
  * Replace standard "exit" command.
@@ -321,7 +327,7 @@ exitcmd(
 	case 2:
 	    if (Tcl_GetIntFromObj(interp, objv[1], &value) != TCL_OK)
 		break;
-	    /* FALLTHROUGH */
+	    // FALLTHROUGH
 	case 1:
 	    tclinfo.exitvalue = value;
 
@@ -402,7 +408,7 @@ buffercmd(
 	Tcl_SetResult(interp, _("invalid buffer number"), TCL_STATIC);
 	return TCL_ERROR;
     }
-    Tcl_ResetResult(interp); /* clear error from Tcl_GetIntFromObj */
+    Tcl_ResetResult(interp); // clear error from Tcl_GetIntFromObj
 
     err = Tcl_GetIndexFromObj(interp, objv[1], bcmdoptions, "option", 0, &idx);
     if (err != TCL_OK)
@@ -576,7 +582,7 @@ bufselfcmd(
 	     * Get line number of last line.
 	     */
 	    opt = 1;
-	    /* fallthrough */
+	    // fallthrough
 	case BUF_COUNT:
 	    /*
 	     * Get number of lines in buffer.
@@ -639,7 +645,7 @@ bufselfcmd(
 			err = TCL_ERROR;
 		}
 	    }
-	    else {  /* objc == 3 */
+	    else {  // objc == 3
 		line = (char *)ml_get_buf(buf, (linenr_T)val1, FALSE);
 		Tcl_SetResult(interp, line, TCL_VOLATILE);
 	    }
@@ -725,7 +731,7 @@ bufselfcmd(
 		}
 		if (i < lc)
 		{
-		    /* append lines */
+		    // append lines
 		    do
 		    {
 			line = Tcl_GetStringFromObj(lv[i], NULL);
@@ -738,22 +744,22 @@ bufselfcmd(
 		}
 		else if (n <= val2)
 		{
-		    /* did not replace all lines, delete */
+		    // did not replace all lines, delete
 		    i = n;
 		    do
 		    {
-			if (ml_delete((linenr_T)i, FALSE) != OK)
+			if (ml_delete((linenr_T)i) != OK)
 			    goto setListError;
 			++n;
 		    } while (n <= val2);
 		}
-		lc -= val2 - val1 + 1;	/* number of lines to be replaced */
+		lc -= val2 - val1 + 1;	// number of lines to be replaced
 		mark_adjust((linenr_T)val1, (linenr_T)val2, (long)MAXLNUM,
 								    (long)lc);
 		changed_lines((linenr_T)val1, 0, (linenr_T)val2 + 1, (long)lc);
 		break;
     setListError:
-		u_undo(1);  /* ??? */
+		u_undo(1);  // ???
 		Tcl_SetResult(interp, _("cannot set line(s)"), TCL_STATIC);
 		err = TCL_ERROR;
 	    }
@@ -790,7 +796,7 @@ bufselfcmd(
 	    }
 	    for (i = 0; i < n; i++)
 	    {
-		ml_delete((linenr_T)val1, FALSE);
+		ml_delete((linenr_T)val1);
 		err = vimerror(interp);
 		if (err != TCL_OK)
 		    break;
@@ -811,9 +817,7 @@ bufselfcmd(
 
 	    pos = NULL;
 	    if (line[0] != '\0'  &&  line[1] == '\0')
-	    {
 		pos = getmark(line[0], FALSE);
-	    }
 	    if (pos == NULL)
 	    {
 		Tcl_SetResult(interp, _("invalid mark name"), TCL_STATIC);
@@ -840,7 +844,7 @@ bufselfcmd(
 
 	case BUF_INSERT:
 	    opt = 1;
-	    /* fallthrough */
+	    // fallthrough
 	case BUF_APPEND:
 	    if (objc != 4)
 	    {
@@ -1080,7 +1084,7 @@ winselfcmd(
 		if (err != TCL_OK)
 		    break;
 	    }
-	    else {  /* objc == 4 */
+	    else {  // objc == 4
 		err = tclgetlinenum(interp, objv[2], &val1, win->w_buffer);
 		if (err != TCL_OK)
 		    break;
@@ -1088,7 +1092,7 @@ winselfcmd(
 		if (err != TCL_OK)
 		    break;
 	    }
-	    /* TODO: should check column */
+	    // TODO: should check column
 	    win->w_cursor.lnum = val1;
 	    win->w_cursor.col = col2vim(val2);
 	    win->w_set_curswant = TRUE;
@@ -1147,9 +1151,9 @@ exprcmd(
     return tclvimexpr(interp, objc, objv, 1);
 }
 
-/****************************************************************************
-  Support functions for Tcl commands
- ****************************************************************************/
+/////////////////////////////////////////////////////////////////////////////
+// Support functions for Tcl commands
+////////////////////////////////////////////////////////////////////////////
 
 /*
  * Get a line number from 'obj' and convert it to vim's range.
@@ -1217,7 +1221,7 @@ tclfindwin(buf_T *buf)
 	if (win->w_buffer == buf)
 	    return win;
     }
-    return curwin;  /* keep current window context */
+    return curwin;  // keep current window context
 }
 
 /*
@@ -1269,7 +1273,7 @@ tcldoexcommand(
 	--emsg_off;
     err = vimerror(interp);
 
-    /* If the ex command created a new Tcl interpreter, remove it */
+    // If the ex command created a new Tcl interpreter, remove it
     if (tclinfo.interp)
 	tcldelthisinterp();
     memcpy(&tclinfo, &saveinfo, sizeof(tcl_info));
@@ -1290,7 +1294,7 @@ tclsetoption(
 {
     int		err, nobjs, idx;
     char_u	*option;
-    int		isnum;
+    getoption_T	gov;
     long	lval;
     char_u	*sval;
     Tcl_Obj	*resobj;
@@ -1307,15 +1311,16 @@ tclsetoption(
 
     option = (char_u *)Tcl_GetStringFromObj(objv[objn], NULL);
     ++objn;
-    isnum = get_option_value(option, &lval, &sval, 0);
+    gov = get_option_value(option, &lval, &sval, NULL, 0);
     err = TCL_OK;
-    switch (isnum)
+    switch (gov)
     {
-	case 0:
+	case gov_string:
 	    Tcl_SetResult(interp, (char *)sval, TCL_VOLATILE);
 	    vim_free(sval);
 	    break;
-	case 1:
+	case gov_bool:
+	case gov_number:
 	    resobj = Tcl_NewLongObj(lval);
 	    Tcl_SetObjResult(interp, resobj);
 	    break;
@@ -1325,9 +1330,9 @@ tclsetoption(
     }
     if (nobjs == 2)
     {
-	if (isnum)
+	if (gov != gov_string)
 	{
-	    sval = NULL;    /* avoid compiler warning */
+	    sval = NULL;    // avoid compiler warning
 	    err = Tcl_GetIndexFromObj(interp, objv[objn], optkw, "", 0, &idx);
 	    if (err != TCL_OK)
 	    {
@@ -1335,17 +1340,19 @@ tclsetoption(
 		err = Tcl_GetLongFromObj(interp, objv[objn], &lval);
 	    }
 	    else
-	    switch (idx)
 	    {
-		case OPT_ON:
-		    lval = 1;
-		    break;
-		case OPT_OFF:
-		    lval = 0;
-		    break;
-		case OPT_TOGGLE:
-		    lval = !lval;
-		    break;
+		switch (idx)
+		{
+		    case OPT_ON:
+			lval = 1;
+			break;
+		    case OPT_OFF:
+			lval = 0;
+			break;
+		    case OPT_TOGGLE:
+			lval = !lval;
+			break;
+		}
 	    }
 	}
 	else
@@ -1382,7 +1389,7 @@ tclvimexpr(
 
 #ifdef FEAT_EVAL
     expr = Tcl_GetStringFromObj(objv[objn], NULL);
-    str = (char *)eval_to_string((char_u *)expr, NULL, TRUE);
+    str = (char *)eval_to_string((char_u *)expr, TRUE);
     if (str == NULL)
 	Tcl_SetResult(interp, _("invalid expression"), TCL_STATIC);
     else
@@ -1412,7 +1419,7 @@ vimerror(Tcl_Interp *interp)
     }
     else if (did_emsg)
     {
-	Tcl_SetResult(interp, _("vim error"), TCL_STATIC);
+	Tcl_SetResult(interp, _("Vim error"), TCL_STATIC);
 	return TCL_ERROR;
     }
     return TCL_OK;
@@ -1442,11 +1449,11 @@ delref(ClientData cref)
     static char *
 tclgetref(
     Tcl_Interp	*interp,
-    void	**refstartP,	/* ptr to w_tcl_ref/b_tcl-ref member of
-				   win_T/buf_T struct */
-    char	*prefix,	/* "win" or "buf" */
-    void	*vimobj,	/* win_T* or buf_T* */
-    Tcl_ObjCmdProc *proc)	/* winselfcmd or bufselfcmd */
+    void	**refstartP,	// ptr to w_tcl_ref/b_tcl-ref member of
+				// win_T/buf_T struct
+    char	*prefix,	// "win" or "buf"
+    void	*vimobj,	// win_T* or buf_T*
+    Tcl_ObjCmdProc *proc)	// winselfcmd or bufselfcmd
 {
     struct ref *ref, *unused = NULL;
     static char name[VARNAME_SIZE];
@@ -1483,7 +1490,7 @@ tclgetref(
 	    (*refstartP) = (void *)ref;
 	}
 
-	/* This might break on some exotic systems... */
+	// This might break on some exotic systems...
 	vim_snprintf(name, sizeof(name), "::vim::%s_%lx",
 					       prefix, (unsigned long)vimobj);
 	cmd = Tcl_CreateObjCommand(interp, name, proc,
@@ -1529,32 +1536,30 @@ tclsetdelcmd(
 	if (reflist->interp == interp && reflist->vimobj == vimobj)
 	{
 	    if (reflist->delcmd)
-	    {
 		Tcl_DecrRefCount(reflist->delcmd);
-	    }
 	    Tcl_IncrRefCount(delcmd);
 	    reflist->delcmd = delcmd;
 	    return TCL_OK;
 	}
 	reflist = reflist->next;
     }
-    /* This should never happen.  Famous last word? */
-    EMSG(_("E280: TCL FATAL ERROR: reflist corrupt!? Please report this to vim-dev@vim.org"));
+    // This should never happen.  Famous last word?
+    iemsg(_(e_tcl_fatal_error_reflist_corrupt_please_report_this));
     Tcl_SetResult(interp, _("cannot register callback command: buffer/window reference not found"), TCL_STATIC);
     return TCL_ERROR;
 }
 
 
-/*******************************************
-    I/O Channel
-********************************************/
+////////////////////////////////////////////
+//    I/O Channel
+////////////////////////////////////////////
 
     static int
 tcl_channel_close(ClientData instance, Tcl_Interp *interp UNUSED)
 {
     int		err = 0;
 
-    /* currently does nothing */
+    // currently does nothing
 
     if (instance != VIMOUT && instance != VIMERR)
     {
@@ -1572,7 +1577,7 @@ tcl_channel_input(
     int		*errptr)
 {
 
-    /* input is currently not supported */
+    // input is currently not supported
 
     Tcl_SetErrno(EINVAL);
     if (errptr)
@@ -1590,10 +1595,9 @@ tcl_channel_output(
     char_u	*str;
     int		result;
 
-    /* The buffer is not guaranteed to be 0-terminated, and we don't if
-     * there is enough room to add a '\0'.  So we have to create a copy
-     * of the buffer...
-     */
+    // The buffer is not guaranteed to be 0-terminated, and we don't if
+    // there is enough room to add a '\0'.  So we have to create a copy
+    // of the buffer...
     str = vim_strnsave((char_u *)buf, bufsiz);
     if (!str)
     {
@@ -1639,43 +1643,43 @@ tcl_channel_gethandle(
 
 static Tcl_ChannelType tcl_channel_type =
 {
-    "vimmessage",	/* typeName */
-    TCL_CHANNEL_VERSION_2, /* version */
-    tcl_channel_close,	/* closeProc */
-    tcl_channel_input,	/* inputProc */
-    tcl_channel_output,	/* outputProc */
-    NULL,		/* seekProc */
-    NULL,		/* setOptionProc */
-    NULL,		/* getOptionProc */
-    tcl_channel_watch,	/* watchProc */
-    tcl_channel_gethandle, /* getHandleProc */
-    NULL,		/* close2Proc */
-    NULL,		/* blockModeProc */
+    "vimmessage",	// typeName
+    TCL_CHANNEL_VERSION_2, // version
+    tcl_channel_close,	// closeProc
+    tcl_channel_input,	// inputProc
+    tcl_channel_output,	// outputProc
+    NULL,		// seekProc
+    NULL,		// setOptionProc
+    NULL,		// getOptionProc
+    tcl_channel_watch,	// watchProc
+    tcl_channel_gethandle, // getHandleProc
+    NULL,		// close2Proc
+    NULL,		// blockModeProc
 #ifdef TCL_CHANNEL_VERSION_2
-    NULL,		/* flushProc */
-    NULL,		/* handlerProc */
+    NULL,		// flushProc
+    NULL,		// handlerProc
 #endif
-/* The following should not be necessary since TCL_CHANNEL_VERSION_2 was
- * set above */
+// The following should not be necessary since TCL_CHANNEL_VERSION_2 was
+// set above
 #ifdef TCL_CHANNEL_VERSION_3
-    NULL,		/* wideSeekProc */
+    NULL,		// wideSeekProc
 #endif
 #ifdef TCL_CHANNEL_VERSION_4
-    NULL,		/* threadActionProc */
+    NULL,		// threadActionProc
 #endif
 #ifdef TCL_CHANNEL_VERSION_5
-    NULL		/* truncateProc */
+    NULL		// truncateProc
 #endif
 };
 
-/**********************************
-  Interface to vim
- **********************************/
+///////////////////////////////////
+// Interface to vim
+//////////////////////////////////
 
     static void
 tclupdatevars(void)
 {
-    char varname[VARNAME_SIZE];	/* must be writeable */
+    char varname[VARNAME_SIZE];	// must be writeable
     char *name;
 
     strcpy(varname, VAR_RANGE1);
@@ -1703,13 +1707,13 @@ tclupdatevars(void)
     static int
 tclinit(exarg_T *eap)
 {
-    char varname[VARNAME_SIZE];	/* Tcl_LinkVar requires writeable varname */
+    char varname[VARNAME_SIZE];	// Tcl_LinkVar requires writeable varname
     char *name;
 
 #ifdef DYNAMIC_TCL
     if (!tcl_enabled(TRUE))
     {
-	EMSG(_("E571: Sorry, this command is disabled: the Tcl library could not be loaded."));
+	emsg(_(e_sorry_this_command_is_disabled_tcl_library_could_not_be_loaded));
 	return FAIL;
     }
 #endif
@@ -1719,9 +1723,9 @@ tclinit(exarg_T *eap)
 	Tcl_Interp *interp;
 	static Tcl_Channel ch1, ch2;
 
-	/* Create replacement channels for stdout and stderr; this has to be
-	 * done each time an interpreter is created since the channels are closed
-	 * when the interpreter is deleted */
+	// Create replacement channels for stdout and stderr; this has to be
+	// done each time an interpreter is created since the channels are closed
+	// when the interpreter is deleted
 	ch1 = Tcl_CreateChannel(&tcl_channel_type, "vimout", VIMOUT, TCL_WRITABLE);
 	ch2 = Tcl_CreateChannel(&tcl_channel_type, "vimerr", VIMERR, TCL_WRITABLE);
 	Tcl_SetStdChannel(ch1, TCL_STDOUT);
@@ -1736,25 +1740,25 @@ tclinit(exarg_T *eap)
 	    return FAIL;
 	}
 #if 0
-	/* VIM sure is interactive */
+	// VIM sure is interactive
 	Tcl_SetVar(interp, "tcl_interactive", "1", TCL_GLOBAL_ONLY);
 #endif
 
 	Tcl_SetChannelOption(interp, ch1, "-buffering", "line");
-#ifdef WIN3264
+#ifdef MSWIN
 	Tcl_SetChannelOption(interp, ch1, "-translation", "lf");
 #endif
 	Tcl_SetChannelOption(interp, ch2, "-buffering", "line");
-#ifdef WIN3264
+#ifdef MSWIN
 	Tcl_SetChannelOption(interp, ch2, "-translation", "lf");
 #endif
 
-	/* replace standard Tcl exit command */
+	// replace standard Tcl exit command
 	Tcl_DeleteCommand(interp, "exit");
 	Tcl_CreateObjCommand(interp, "exit", exitcmd,
 	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);
 
-	/* new commands, in ::vim namespace */
+	// new commands, in ::vim namespace
 	Tcl_CreateObjCommand(interp, "::vim::buffer", buffercmd,
 	    (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);
 	Tcl_CreateObjCommand(interp, "::vim::window", windowcmd,
@@ -1768,12 +1772,12 @@ tclinit(exarg_T *eap)
 	Tcl_CreateObjCommand(interp, "::vim::expr", exprcmd,
 	   (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);
 
-	/* "lbase" variable */
+	// "lbase" variable
 	tclinfo.lbase = 1;
 	strcpy(varname, VAR_LBASE);
 	Tcl_LinkVar(interp, varname, (char *)&tclinfo.lbase, TCL_LINK_INT);
 
-	/* "range" variable */
+	// "range" variable
 	tclinfo.range_start = eap->line1;
 	strcpy(varname, VAR_RANGE1);
 	Tcl_LinkVar(interp, varname, (char *)&tclinfo.range_start, TCL_LINK_INT|TCL_LINK_READ_ONLY);
@@ -1783,7 +1787,7 @@ tclinit(exarg_T *eap)
 	strcpy(varname, VAR_RANGE3);
 	Tcl_LinkVar(interp, varname, (char *)&tclinfo.range_end, TCL_LINK_INT|TCL_LINK_READ_ONLY);
 
-	/* "current" variable */
+	// "current" variable
 	tclinfo.curbuf = Tcl_Alloc(VARNAME_SIZE);
 	tclinfo.curwin = Tcl_Alloc(VARNAME_SIZE);
 	name = tclgetbuffer(interp, curbuf);
@@ -1799,7 +1803,7 @@ tclinit(exarg_T *eap)
     }
     else
     {
-	/* Interpreter already exists, just update variables */
+	// Interpreter already exists, just update variables
 	tclinfo.range_start = row2tcl(eap->line1);
 	tclinfo.range_end = row2tcl(eap->line2);
 	tclupdatevars();
@@ -1817,11 +1821,11 @@ tclerrmsg(char *text)
     while ((next=strchr(text, '\n')))
     {
 	*next++ = '\0';
-	EMSG(text);
+	emsg(text);
 	text = next;
     }
     if (*text)
-	EMSG(text);
+	emsg(text);
 }
 
     static void
@@ -1832,11 +1836,11 @@ tclmsg(char *text)
     while ((next=strchr(text, '\n')))
     {
 	*next++ = '\0';
-	MSG(text);
+	msg(text);
 	text = next;
     }
     if (*text)
-	MSG(text);
+	msg(text);
 }
 
     static void
@@ -1845,15 +1849,14 @@ tcldelthisinterp(void)
     if (!Tcl_InterpDeleted(tclinfo.interp))
 	Tcl_DeleteInterp(tclinfo.interp);
     Tcl_Release(tclinfo.interp);
-    /* The interpreter is now gets deleted.  All registered commands (esp.
-     * window and buffer commands) are deleted, triggering their deletion
-     * callback, which deletes all refs pointing to this interpreter.
-     * We could garbage-collect the unused ref structs in all windows and
-     * buffers, but unless the user creates hundreds of sub-interpreters
-     * all referring to lots of windows and buffers, this is hardly worth
-     * the effort.  Unused refs are recycled by other interpreters, and
-     * all refs are free'd when the window/buffer gets closed by vim.
-     */
+    // The interpreter is now gets deleted.  All registered commands (esp.
+    // window and buffer commands) are deleted, triggering their deletion
+    // callback, which deletes all refs pointing to this interpreter.
+    // We could garbage-collect the unused ref structs in all windows and
+    // buffers, but unless the user creates hundreds of sub-interpreters
+    // all referring to lots of windows and buffers, this is hardly worth
+    // the effort.  Unused refs are recycled by other interpreters, and
+    // all refs are free'd when the window/buffer gets closed by vim.
 
     tclinfo.interp = NULL;
     Tcl_Free(tclinfo.curbuf);
@@ -1866,15 +1869,15 @@ tclexit(int error)
 {
     int newerr = OK;
 
-    if (Tcl_InterpDeleted(tclinfo.interp)     /* True if we intercepted Tcl's exit command */
+    if (Tcl_InterpDeleted(tclinfo.interp)     // True if we intercepted Tcl's exit command
 #if (TCL_MAJOR_VERSION == 8 && TCL_MINOR_VERSION >= 5) || TCL_MAJOR_VERSION > 8
-	|| Tcl_LimitExceeded(tclinfo.interp)  /* True if the interpreter cannot continue */
+	|| Tcl_LimitExceeded(tclinfo.interp)  // True if the interpreter cannot continue
 #endif
 	)
     {
 	char buf[50];
 
-	sprintf(buf, _("E572: exit code %d"), tclinfo.exitvalue);
+	sprintf(buf, _(e_exit_code_nr), tclinfo.exitvalue);
 	tclerrmsg(buf);
 	if (tclinfo.exitvalue == 0)
 	{
@@ -1958,7 +1961,7 @@ ex_tcldo(exarg_T *eap)
 {
     char	*script, *line;
     int		err, rs, re, lnum;
-    char	var_lnum[VARNAME_SIZE]; /* must be writeable memory */
+    char	var_lnum[VARNAME_SIZE]; // must be writeable memory
     char	var_line[VARNAME_SIZE];
     linenr_T	first_line = 0;
     linenr_T	last_line = 0;
@@ -2039,7 +2042,7 @@ tcldelallrefs(struct ref *ref)
     char	*result;
 
 #ifdef DYNAMIC_TCL
-    /* TODO: this code currently crashes Vim on exit */
+    // TODO: this code currently crashes Vim on exit
     if (exiting)
 	return;
 #endif
@@ -2074,7 +2077,7 @@ tcl_buffer_free(buf_T *buf)
     struct ref *reflist;
 
 #ifdef DYNAMIC_TCL
-    if (!stubs_initialized)	/* Not using Tcl, nothing to do. */
+    if (!stubs_initialized)	// Not using Tcl, nothing to do.
 	return;
 #endif
 
@@ -2093,7 +2096,7 @@ tcl_window_free(win_T *win)
     struct ref *reflist;
 
 #ifdef DYNAMIC_TCL
-    if (!stubs_initialized)	/* Not using Tcl, nothing to do. */
+    if (!stubs_initialized)	// Not using Tcl, nothing to do.
 	return;
 #endif
 
@@ -2106,4 +2109,4 @@ tcl_window_free(win_T *win)
     }
 }
 
-/* The End */
+// The End
